package org.greenstone.gsdl3.gs3build;

import org.greenstone.gsdl3.gs3build.doctypes.*;
import org.greenstone.gsdl3.gs3build.indexers.*;
import org.greenstone.gsdl3.gs3build.extractor.*;

import java.io.File;
import java.util.List;

public class BuildManager 
{
  CollectionManager collectionManager;
  RecogniserManager recogniserManager;
  IndexerManager    indexerManager;
  ExtractorManager  extractorManager;
  DocumentList      docList;
  List              inputRoots;
  String            outputDir;
	
    public BuildManager(List inputRoots, String collection, String outputDir)
    {
      // get the collection configuration information
      this.collectionManager = new CollectionManager(collection);

      // set up the list of input directories
      if (inputRoots.size() == 0) {
	inputRoots.add(this.collectionManager.getImportDirectory());
	System.out.println("Adding collection import directory to input list");
      }
      this.docList = new DocumentList(new DocumentIDFactory(this.collectionManager));
      this.inputRoots = inputRoots;

      // set up the output directory
      this.outputDir  = outputDir != null ? outputDir : this.collectionManager.getBuildDirectory();
      File outputDirFile = new File(this.outputDir);
      if (!outputDirFile.exists() || !outputDirFile.isDirectory())
      { System.err.println("Unable to open directory " + this.outputDir + " for writing");
	System.exit(1);
      }
      System.out.println("Output directory set to " + this.outputDir);

      // initialise any recognisers
      this.recogniserManager = new RecogniserManager();
      RecogniserInterface recogniser = new HTMLRecogniser(docList);
      recogniserManager.addRecogniser(recogniser);
      recogniser = new TextRecogniser(docList);
      recogniserManager.addRecogniser(recogniser);
      
      // set up the extractors
      this.extractorManager  = new ExtractorManager(this.docList);

      // set up the enrichers

      // set up the classifiers

      // set up the indexers
      this.indexerManager = new IndexerManager(this.docList);
      IndexerInterface iface = new MGIndexer();
      iface.configure(this.outputDir);
      this.indexerManager.addIndexer(iface);
    }

    public String getLastBuildSequence()
    { return this.collectionManager.getCollectionMetadata("gsdl3", "buildsequence");
    }

	public void run()
	{
		this.collectionManager.startBuild();

		for (int i = 0; i < this.inputRoots.size(); i ++)
		{ FileCrawler fileCrawler = new FileCrawler(new File((String) this.inputRoots.get(i)), recogniserManager);

			fileCrawler.crawl();
		}
		this.indexerManager.indexDocuments();

		this.docList.writeDocuments(new File(outputDir));
		this.docList.writeSQLDocuments(this.collectionManager.getDatabase());

		this.collectionManager.endBuild();
	}

	public void addInputDirectory(String inputDir)
	{	this.inputRoots.add(inputDir);
	}
}
